@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////
 //                           **** WAVPACK ****                            //
 //                  Hybrid Lossless Wavefile Compressor                   //
-//              Copyright (c) 1998 - 2009 Conifer Software.               //
+//                Copyright (c) 1998 - 2015 David Bryant.                 //
 //                          All Rights Reserved.                          //
 //      Distributed under the BSD Software License (see license.txt)      //
 ////////////////////////////////////////////////////////////////////////////
@@ -17,6 +17,7 @@
 // ReplayGain's [somewhat outdated] website: http://replaygain.org/
 
 #if defined(WIN32)
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <io.h>
 #else
@@ -66,11 +67,18 @@ static char *strdup (const char *s)
 
 static const char *sign_on = "\n"
 " WVGAIN  ReplayGain Scanner/Tagger for WavPack  %s Version %s\n"
-" Copyright (c) 2005 - 2009 Conifer Software.  All Rights Reserved.\n\n";
+" Copyright (c) 2005 - 2015 David Bryant.  All Rights Reserved.\n\n";
+
+static const char *version_warning = "\n"
+" WARNING: WVGAIN using libwavpack version %s, expected %s (see README)\n\n";
 
 static const char *usage =
 " Usage:   WVGAIN [-options] [@]infile[.wv] [...]\n"
+#if defined (WIN32) || defined (__OS2__)
 "             (infiles may contain wildcards: ?,*)\n\n"
+#else
+"             (multiple input files are allowed)\n\n"
+#endif
 " Options: -a  = album mode (all files scanned are considered an album)\n"
 "          -c  = clean ReplayGain values from all files (no analysis)\n"
 "          -d  = display calculated values only (no files are modified)\n"
@@ -80,8 +88,14 @@ static const char *usage =
 #endif
 "          -n  = new files only (skip files with track info, or album\n"
 "                 info if album mode specified)\n"
+"          -q  = quiet (keep console output to a minimum)\n"
+"          -v  = write the version to stdout\n"
 "          -s  = show stored values only (no analysis)\n"
-"          -q  = quiet (keep console output to a minimum)\n\n"
+#if defined (WIN32)
+"          -z  = don't set console title to indicate progress\n\n"
+#else
+"          -z1 = set console title to indicate progress\n\n"
+#endif
 " Web:     Visit www.wavpack.com for latest version and info\n";
 
 // this global is used to indicate the special "debug" mode where extra debug messages
@@ -92,7 +106,7 @@ int debug_logging_mode;
 #define HISTOGRAM_SLOTS 12000
 static uint32_t track_histogram [HISTOGRAM_SLOTS], album_histogram [HISTOGRAM_SLOTS];
 
-static char album_mode, clean_mode, display_mode, ignore_wvc, quiet_mode, show_mode, new_mode;
+static char album_mode, clean_mode, display_mode, ignore_wvc, quiet_mode, show_mode, new_mode, set_console_title;
 static int num_files, file_index;
 
 /////////////////////////// local function declarations ///////////////////////
@@ -147,6 +161,10 @@ int main (argc, argv) int argc; char **argv;
     }
 #endif
 
+#if defined (WIN32)
+   set_console_title = 1;      // on Windows, we default to messing with the console title
+#endif                          // on Linux, this is considered uncool to do by default
+
     // loop through command-line arguments
 
     while (--argc) {
@@ -157,6 +175,12 @@ int main (argc, argv) int argc; char **argv;
 #endif
             while (*++*argv)
                 switch (**argv) {
+
+                    case 'V': case 'v':
+                        printf ("wvgain %s\n", PACKAGE_VERSION);
+                        printf ("libwavpack %s\n", WavpackGetLibraryVersionString ());
+                        return 0;
+
                     case 'A': case 'a':
                         album_mode = 1;
                         break;
@@ -186,6 +210,11 @@ int main (argc, argv) int argc; char **argv;
                         quiet_mode = 1;
                         break;
 
+                    case 'Z': case 'z':
+                        set_console_title = (char) strtol (++*argv, argv, 10);
+                        --*argv;
+                        break;
+
                     case 'I': case 'i':
                         ignore_wvc = 1;
                         break;
@@ -222,7 +251,9 @@ int main (argc, argv) int argc; char **argv;
         ++error_count;
     }
 
-    if (!quiet_mode && !error_count)
+    if (strcmp (WavpackGetLibraryVersionString (), PACKAGE_VERSION))
+        fprintf (stderr, version_warning, WavpackGetLibraryVersionString (), PACKAGE_VERSION);
+    else if (!quiet_mode && !error_count)
         fprintf (stderr, sign_on, VERSION_OS, WavpackGetLibraryVersionString ());
 
     if (!num_files) {
@@ -339,7 +370,7 @@ int main (argc, argv) int argc; char **argv;
             if (check_break ())
                 break;
 
-            if (num_files > 1)
+            if (num_files > 1 && !quiet_mode)
                 fprintf (stderr, "\n%s:\n", matches [file_index]);
 
             if (new_mode) {
@@ -348,21 +379,22 @@ int main (argc, argv) int argc; char **argv;
 
                 wpc = WavpackOpenFileInput (matches [file_index], error, OPEN_TAGS, 0);
 
-                if (wpc && WavpackGetTagItem (wpc, album_mode ? "replaygain_album_gain" : "replaygain_track_gain", NULL, 0)) {
+                if (wpc) {
+                    int alreadyHasTag = WavpackGetTagItem (wpc, album_mode ? "replaygain_album_gain" : "replaygain_track_gain", NULL, 0);
                     WavpackCloseFile (wpc);
 
-                    if (album_mode) {
-                        error_line ("ReplayGain album information already present...aborting");
-                        result = HARD_ERROR;
-                        break;
-                    }
-                    else {
-                        error_line ("ReplayGain track information already present...skipping");
-                        continue;
+                    if (alreadyHasTag) {
+                        if (album_mode) {
+                            error_line ("ReplayGain album information already present...aborting");
+                            result = HARD_ERROR;
+                            break;
+                        }
+                        else {
+                            error_line ("ReplayGain track information already present...skipping");
+                            continue;
+                        }
                     }
                 }
-
-                WavpackCloseFile (wpc);
             }
 
             result = analyze_file (matches [file_index], track_histogram, &track_peak);
@@ -423,7 +455,7 @@ int main (argc, argv) int argc; char **argv;
                 if (check_break ())
                     break;
 
-                if (num_files > 1)
+                if (num_files > 1 && !quiet_mode)
                     fprintf (stderr, "\n%s:\n", matches [file_index]);
 
                 if (show_mode)
@@ -460,6 +492,9 @@ int main (argc, argv) int argc; char **argv;
     error_line ("malloc_count = %d", dump_alloc ());
 #endif
 
+    if (set_console_title)
+        DoSetConsoleTitle ("WvGain Completed");
+
     return error_count ? 1 : 0;
 }
 
@@ -570,7 +605,11 @@ static int analyze_file (char *infilename, uint32_t *histogram, float *peak)
             break;
 
         if (check_break ()) {
+#if defined(WIN32)
             fprintf (stderr, "^C\n");
+#else
+            fprintf (stderr, "\n");
+#endif
             result = HARD_ERROR;
             break;
         }
@@ -639,8 +678,10 @@ static int update_file (char *infilename, float track_gain, float track_peak, fl
         if (WavpackDeleteTagItem (wpc, "replaygain_album_peak"))
             ++items_removed;
 
-        if (!quiet_mode && items_removed) {
-            error_line ("%d ReplayGain values cleaned", items_removed);
+        if (items_removed) {
+            if (!quiet_mode)
+                error_line ("%d ReplayGain values cleaned", items_removed);
+
             write_tag = TRUE;
         }
         else
@@ -1153,9 +1194,9 @@ void display_progress (double file_progress)
 {
     char title [40];
 
-    file_progress = (file_index + file_progress) / num_files;
-    sprintf (title, "%d%% (WvGain)", (int) ((file_progress * 100.0) + 0.5));
-#if defined(WIN32)
-    SetConsoleTitle (title);
-#endif
+    if (set_console_title) {
+        file_progress = (file_index + file_progress) / num_files;
+        sprintf (title, "%d%% (WvGain)", (int) ((file_progress * 100.0) + 0.5));
+        DoSetConsoleTitle (title);
+    }
 }
