@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////
 //                           **** WAVPACK ****                            //
 //                  Hybrid Lossless Wavefile Compressor                   //
-//              Copyright (c) 1998 - 2013 Conifer Software.               //
+//                Copyright (c) 1998 - 2015 David Bryant.                 //
 //                          All Rights Reserved.                          //
 //      Distributed under the BSD Software License (see license.txt)      //
 ////////////////////////////////////////////////////////////////////////////
@@ -68,7 +68,10 @@ static char *strdup (const char *s)
 
 static const char *sign_on = "\n"
 " WVUNPACK  Hybrid Lossless Audio Decompressor  %s Version %s\n"
-" Copyright (c) 1998 - 2013 Conifer Software.  All Rights Reserved.\n\n";
+" Copyright (c) 1998 - 2015 David Bryant.  All Rights Reserved.\n\n";
+
+static const char *version_warning = "\n"
+" WARNING: WVUNPACK using libwavpack version %s, expected %s (see README)\n\n";
 
 static const char *usage =
 #if defined (WIN32)
@@ -76,8 +79,8 @@ static const char *usage =
 "             (infile may contain wildcards: ?,*)\n\n"
 #else
 " Usage:   WVUNPACK [-options] infile[.wv]|- [...] [-o outfile[.wav]|outpath|-]\n"
-#endif
 "             (multiple input files are allowed)\n\n"
+#endif
 " Options: -b  = blindly decode all stream blocks & ignore length info\n"
 "          -c  = extract cuesheet only to stdout (no audio decode)\n"
 "               (note: equivalent to -x \"cuesheet\")\n"
@@ -115,7 +118,11 @@ static const char *usage =
 "                %t = tag field name (note: comes from data for binary tags)\n"
 "                %e = extension from binary tag source file, or 'txt' for text tag\n"
 "          -y  = yes to overwrite warning (use with caution!)\n"
+#if defined (WIN32)
 "          -z  = don't set console title to indicate progress\n\n"
+#else
+"          -z1 = set console title to indicate progress\n\n"
+#endif
 " Web:     Visit www.wavpack.com for latest version and info\n";
 
 // this global is used to indicate the special "debug" mode where extra debug messages
@@ -124,7 +131,7 @@ static const char *usage =
 int debug_logging_mode;
 
 static char overwrite_all, delete_source, raw_decode, no_utf8_convert, no_audio_decode, file_info,
-    summary, ignore_wvc, quiet_mode, calc_md5, copy_time, blind_decode, wav_decode, no_console_title;
+    summary, ignore_wvc, quiet_mode, calc_md5, copy_time, blind_decode, wav_decode, set_console_title;
 
 static int num_files, file_index, outbuf_k;
 
@@ -188,6 +195,10 @@ int main (argc, argv) int argc; char **argv;
     }
 #endif
 
+#if defined (WIN32)
+   set_console_title = 1;      // on Windows, we default to messing with the console title
+#endif                          // on Linux, this is considered uncool to do by default
+
     // loop through command-line arguments
 
     while (--argc) {
@@ -200,12 +211,12 @@ int main (argc, argv) int argc; char **argv;
 
             if (!strcmp (long_option, "help")) {                        // --help
                 printf ("%s", usage);
-                return 1;
+                return 0;
             }
             else if (!strcmp (long_option, "version")) {                // --version
                 printf ("wvunpack %s\n", PACKAGE_VERSION);
                 printf ("libwavpack %s\n", WavpackGetLibraryVersionString ());
-                return 1;
+                return 0;
             }
             else if (!strcmp (long_option, "no-utf8-convert"))          // --no-utf8-convert
                 no_utf8_convert = 1;
@@ -321,7 +332,8 @@ int main (argc, argv) int argc; char **argv;
                         break;
 
                     case 'Z': case 'z':
-                        no_console_title = 1;
+                        set_console_title = (char) strtol (++*argv, argv, 10);
+                        --*argv;
                         break;
 
                     case 'X': case 'x':
@@ -437,7 +449,9 @@ int main (argc, argv) int argc; char **argv;
         ++error_count;
     }
 
-    if (!quiet_mode && !error_count)
+    if (strcmp (WavpackGetLibraryVersionString (), PACKAGE_VERSION))
+        fprintf (stderr, version_warning, WavpackGetLibraryVersionString (), PACKAGE_VERSION);
+    else if (!quiet_mode && !error_count)
         fprintf (stderr, sign_on, VERSION_OS, WavpackGetLibraryVersionString ());
 
     if (!num_files) {
@@ -658,7 +672,7 @@ int main (argc, argv) int argc; char **argv;
     error_line ("malloc_count = %d", dump_alloc ());
 #endif
 
-    if (!no_console_title)
+    if (set_console_title)
         DoSetConsoleTitle ("WvUnpack Completed");
 
     return error_count ? 1 : 0;
@@ -741,7 +755,7 @@ static FILE *open_output_file (char *filename, char **tempfilename)
             if (!overwrite_all) {
                 fprintf (stderr, "overwrite %s (yes/no/all)? ", FN_FIT (filename));
 
-                if (!no_console_title)
+                if (set_console_title)
                     DoSetConsoleTitle ("overwrite?");
 
                 switch (yna ()) {
@@ -818,7 +832,7 @@ static int unpack_file (char *infilename, char *outfilename)
     uint32_t skip_sample_index = 0, until_samples_total = 0;
     unsigned char *output_buffer = NULL, *output_pointer = NULL;
     double dtime, progress = -1.0;
-    char *outfilename_temp;
+    char *outfilename_temp = NULL;
     MD5_CTX md5_context;
     WavpackContext *wpc;
     int32_t *temp_buffer;
@@ -1354,7 +1368,7 @@ static int do_tag_extractions (WavpackContext *wpc, char *outfilename)
                 DoCloseHandle (outfile);
                 fprintf (stderr, "overwrite %s (yes/no/all)? ", FN_FIT (full_filename));
 
-                if (!no_console_title)
+                if (set_console_title)
                     DoSetConsoleTitle ("overwrite?");
 
                 switch (yna ()) {
@@ -2054,7 +2068,7 @@ void display_progress (double file_progress)
 {
     char title [40];
 
-    if (!no_console_title) {
+    if (set_console_title) {
         file_progress = (file_index + file_progress) / num_files;
         sprintf (title, "%d%% (WvUnpack)", (int) ((file_progress * 100.0) + 0.5));
         DoSetConsoleTitle (title);
