@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////
 //                           **** WAVPACK ****                            //
 //                  Hybrid Lossless Wavefile Compressor                   //
-//              Copyright (c) 1998 - 2006 Conifer Software.               //
+//              Copyright (c) 1998 - 2013 Conifer Software.               //
 //                          All Rights Reserved.                          //
 //      Distributed under the BSD Software License (see license.txt)      //
 ////////////////////////////////////////////////////////////////////////////
@@ -10,27 +10,29 @@
 
 // This module handles the "extra" mode for mono files.
 
-#include "wavpack_local.h"
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
 
+#include "wavpack_local.h"
+
+// This flag causes this module to take into account the size of the header
+// (which grows with more decorrelation passes) when making decisions about
+// adding additional passes (as opposed to just considering the resulting
+// magnitude of the residuals). With small blocks this seems to work correctly,
+// but with longer blocks it seems to actually hurt compression (for reasons I
+// cannot explain), so it's disabled by default
+
+//#define USE_OVERHEAD
+
+// If the log2 value of any sample in a buffer being scanned exceeds this value,
+// we abandon that configuration. This prevents us from going down paths that
+// are wildly unstable.
+
 #define LOG_LIMIT 6912
-//#define EXTRA_DUMP
 
-#ifdef DEBUG_ALLOC
-#define malloc malloc_db
-#define realloc realloc_db
-#define free free_db
-void *malloc_db (uint32_t size);
-void *realloc_db (void *ptr, uint32_t size);
-void free_db (void *ptr);
-int32_t dump_alloc (void);
-#endif
-
-//////////////////////////////// local tables ///////////////////////////////
+//#define EXTRA_DUMP        // dump generated filter data  error_line()
 
 typedef struct {
     int32_t *sampleptrs [MAX_NTERMS+2];
@@ -184,6 +186,18 @@ static void decorr_mono_buffer (int32_t *samples, int32_t *outsamples, uint32_t
     decorr_mono_pass (samples, outsamples, num_samples, &dp, 1);
 }
 
+static int log2overhead (int first_term, int num_terms)
+{
+#ifdef USE_OVERHEAD
+    if (first_term > MAX_TERM)
+        return (4 + num_terms * 2) << 11;
+    else
+        return (2 + num_terms * 2) << 11;
+#else
+    return 0;
+#endif
+}
+
 static void recurse_mono (WavpackContext *wpc, WavpackExtraInfo *info, int depth, int delta, uint32_t input_bits)
 {
     WavpackStream *wps = wpc->streams [wpc->current_stream];
@@ -211,7 +225,10 @@ static void recurse_mono (WavpackContext *wpc, WavpackExtraInfo *info, int depth
         info->dps [depth].term = term;
         info->dps [depth].delta = delta;
         decorr_mono_buffer (samples, outsamples, wps->wphdr.block_samples, info->dps, depth);
-        bits = log2buffer (outsamples, wps->wphdr.block_samples, info->log_limit);
+        bits = LOG2BUFFER (outsamples, wps->wphdr.block_samples, info->log_limit);
+
+        if (bits != (uint32_t) -1)
+            bits += log2overhead (info->dps [0].term, depth + 1);
 
         if (bits < info->best_bits) {
             info->best_bits = bits;
@@ -273,7 +290,10 @@ static void delta_mono (WavpackContext *wpc, WavpackExtraInfo *info)
             decorr_mono_buffer (info->sampleptrs [i], info->sampleptrs [i+1], wps->wphdr.block_samples, info->dps, i);
         }
 
-        bits = log2buffer (info->sampleptrs [i], wps->wphdr.block_samples, info->log_limit);
+        bits = LOG2BUFFER (info->sampleptrs [i], wps->wphdr.block_samples, info->log_limit);
+
+        if (bits != (uint32_t) -1)
+            bits += log2overhead (wps->decorr_passes [0].term, i);
 
         if (bits < info->best_bits) {
             lower = TRUE;
@@ -295,7 +315,10 @@ static void delta_mono (WavpackContext *wpc, WavpackExtraInfo *info)
             decorr_mono_buffer (info->sampleptrs [i], info->sampleptrs [i+1], wps->wphdr.block_samples, info->dps, i);
         }
 
-        bits = log2buffer (info->sampleptrs [i], wps->wphdr.block_samples, info->log_limit);
+        bits = LOG2BUFFER (info->sampleptrs [i], wps->wphdr.block_samples, info->log_limit);
+
+        if (bits != (uint32_t) -1)
+            bits += log2overhead (wps->decorr_passes [0].term, i);
 
         if (bits < info->best_bits) {
             info->best_bits = bits;
@@ -336,7 +359,10 @@ static void sort_mono (WavpackContext *wpc, WavpackExtraInfo *info)
             for (i = ri; i < info->nterms && wps->decorr_passes [i].term; ++i)
                 decorr_mono_buffer (info->sampleptrs [i], info->sampleptrs [i+1], wps->wphdr.block_samples, info->dps, i);
 
-            bits = log2buffer (info->sampleptrs [i], wps->wphdr.block_samples, info->log_limit);
+            bits = LOG2BUFFER (info->sampleptrs [i], wps->wphdr.block_samples, info->log_limit);
+
+            if (bits != (uint32_t) -1)
+                bits += log2overhead (wps->decorr_passes [0].term, i);
 
             if (bits < info->best_bits) {
                 reversed = TRUE;
@@ -387,12 +413,13 @@ static void analyze_mono (WavpackContext *wpc, int32_t *samples, int do_samples)
     for (i = 0; i < info.nterms && info.dps [i].term; ++i)
         decorr_mono_pass (info.sampleptrs [i], info.sampleptrs [i + 1], wps->wphdr.block_samples, info.dps + i, 1);
 
-    info.best_bits = log2buffer (info.sampleptrs [info.nterms], wps->wphdr.block_samples, 0) * 1;
+    info.best_bits = LOG2BUFFER (info.sampleptrs [info.nterms], wps->wphdr.block_samples, 0) * 1;
+    info.best_bits += log2overhead (info.dps [0].term, i);
     memcpy (info.sampleptrs [info.nterms + 1], info.sampleptrs [i], wps->wphdr.block_samples * 4);
 
     if (wpc->config.extra_flags & EXTRA_BRANCHES)
         recurse_mono (wpc, &info, 0, (int) floor (wps->delta_decay + 0.5),
-            log2buffer (info.sampleptrs [0], wps->wphdr.block_samples, 0));
+            LOG2BUFFER (info.sampleptrs [0], wps->wphdr.block_samples, 0));
 
     if (wpc->config.extra_flags & EXTRA_SORT_FIRST)
         sort_mono (wpc, &info);
@@ -425,6 +452,7 @@ static void analyze_mono (WavpackContext *wpc, int32_t *samples, int do_samples)
 static void mono_add_noise (WavpackStream *wps, int32_t *lptr, int32_t *rptr)
 {
     int shaping_weight, new = wps->wphdr.flags & NEW_SHAPING;
+    short *shaping_array = wps->dc.shaping_array;
     int32_t error = 0, temp, cnt;
 
     scan_word (wps, rptr, wps->wphdr.block_samples, -1);
@@ -432,7 +460,11 @@ static void mono_add_noise (WavpackStream *wps, int32_t *lptr, int32_t *rptr)
 
     if (wps->wphdr.flags & HYBRID_SHAPE) {
         while (cnt--) {
-            shaping_weight = (wps->dc.shaping_acc [0] += wps->dc.shaping_delta [0]) >> 16;
+            if (shaping_array)
+                shaping_weight = *shaping_array++;
+            else
+                shaping_weight = (wps->dc.shaping_acc [0] += wps->dc.shaping_delta [0]) >> 16;
+
             temp = -apply_weight (shaping_weight, error);
 
             if (new && shaping_weight < 0 && temp) {
@@ -448,7 +480,8 @@ static void mono_add_noise (WavpackStream *wps, int32_t *lptr, int32_t *rptr)
             rptr++;
         }
 
-        wps->dc.shaping_acc [0] -= wps->dc.shaping_delta [0] * wps->wphdr.block_samples;
+        if (!shaping_array)
+            wps->dc.shaping_acc [0] -= wps->dc.shaping_delta [0] * wps->wphdr.block_samples;
     }
     else
         while (cnt--) {
@@ -468,6 +501,12 @@ void execute_mono (WavpackContext *wpc, int32_t *samples, int no_history, int do
     uint32_t best_size = (uint32_t) -1, size;
     int log_limit, pi, i;
 
+#ifdef SKIP_DECORRELATION
+    CLEAR (wps->decorr_passes);
+    wps->num_terms = 0;
+    return;
+#endif
+
     for (i = 0; i < num_samples; ++i)
         if (samples [i])
             break;
@@ -521,7 +560,7 @@ void execute_mono (WavpackContext *wpc, int32_t *samples, int no_history, int do
         wps->best_decorr = wps->mask_decorr = 0;
 
     for (pi = 0; pi < wps->num_passes;) {
-        WavpackDecorrSpec *wpds;
+        const WavpackDecorrSpec *wpds;
         int nterms, c, j;
 
         if (!pi)
@@ -539,7 +578,7 @@ void execute_mono (WavpackContext *wpc, int32_t *samples, int no_history, int do
         }
 
         wpds = &wps->decorr_specs [c];
-        nterms = (int) strlen (wpds->terms);
+        nterms = (int) strlen ((char *) wpds->terms);
 
         while (1) {
         memcpy (temp_buffer [0], noisy_buffer ? noisy_buffer : samples, buf_size);
@@ -566,7 +605,7 @@ void execute_mono (WavpackContext *wpc, int32_t *samples, int no_history, int do
             decorr_mono_pass (temp_buffer [j&1], temp_buffer [~j&1], num_samples, &temp_decorr_pass, 1);
         }
 
-        size = log2buffer (temp_buffer [j&1], num_samples, log_limit);
+        size = LOG2BUFFER (temp_buffer [j&1], num_samples, log_limit);
 
         if (size == (uint32_t) -1 && nterms)
             nterms >>= 1;
@@ -574,6 +613,8 @@ void execute_mono (WavpackContext *wpc, int32_t *samples, int no_history, int do
             break;
         }
 
+        size += log2overhead (wpds->terms [0], nterms);
+
         if (size < best_size) {
             memcpy (best_buffer, temp_buffer [j&1], buf_size);
             memcpy (wps->decorr_passes, save_decorr_passes, sizeof (struct decorr_pass) * MAX_NTERMS);
