@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////
 //                           **** WAVPACK ****                            //
 //                  Hybrid Lossless Wavefile Compressor                   //
-//              Copyright (c) 1998 - 2006 Conifer Software.               //
+//              Copyright (c) 1998 - 2013 Conifer Software.               //
 //                          All Rights Reserved.                          //
 //      Distributed under the BSD Software License (see license.txt)      //
 ////////////////////////////////////////////////////////////////////////////
@@ -12,387 +12,22 @@
 // not including "raw" files. As these modes are all obsolete and are no
 // longer written, this code will not be fully documented other than the
 // global functions. However, full documenation is provided in the version
-// 3.97 source code.
+// 3.97 source code. Note that this module does only the low-level sample
+// unpacking; the actual opening of the file (and obtaining information
+// from it) is handled in the unpack3_open.c module.
 
 #include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
-#include <math.h>
 
 #include "wavpack_local.h"
 #include "unpack3.h"
 
 #define ATTEMPT_ERROR_MUTING
 
-#ifdef DEBUG_ALLOC
-#define malloc malloc_db
-#define realloc realloc_db
-#define free free_db
-void *malloc_db (uint32_t size);
-void *realloc_db (void *ptr, uint32_t size);
-void free_db (void *ptr);
-int32_t dump_alloc (void);
-#endif
-
-static void unpack_init3 (WavpackStream3 *wps);
 static int bs_open_read3 (Bitstream3 *bs, WavpackStreamReader *reader, void *id);
-static void bs_close_read3 (Bitstream3 *bs);
-#ifndef NO_SEEKING
-static void bs_restore3 (Bitstream3 *bs);
-#endif
-
-// This provides an extension to the WavpackOpenFileRead () function contained
-// in the wputils.c module. It is assumed that an 'R' had been read as the
-// first character of the file/stream (indicating a non-raw pre version 4.0
-// WavPack file) and had been pushed back onto the stream (or simply seeked
-// back to).
-
-WavpackContext *open_file3 (WavpackContext *wpc, char *error)
-{
-    RiffChunkHeader RiffChunkHeader;
-    ChunkHeader ChunkHeader;
-    WavpackHeader3 wphdr;
-    WavpackStream3 *wps;
-    WaveHeader3 wavhdr;
-
-    CLEAR (wavhdr);
-    wpc->stream3 = wps = (WavpackStream3 *) malloc (sizeof (WavpackStream3));
-    CLEAR (*wps);
-
-    if (wpc->reader->read_bytes (wpc->wv_in, &RiffChunkHeader, sizeof (RiffChunkHeader)) !=
-        sizeof (RiffChunkHeader)) {
-            if (error) strcpy (error, "not a valid WavPack file!");
-            return WavpackCloseFile (wpc);
-    }
-
-    if (!strncmp (RiffChunkHeader.ckID, "RIFF", 4) && !strncmp (RiffChunkHeader.formType, "WAVE", 4)) {
-
-        if (wpc->open_flags & OPEN_WRAPPER) {
-            wpc->wrapper_data = malloc (wpc->wrapper_bytes = sizeof (RiffChunkHeader));
-            memcpy (wpc->wrapper_data, &RiffChunkHeader, sizeof (RiffChunkHeader));
-        }
-
-    // If the first chunk is a wave RIFF header, then read the various chunks
-    // until we get to the "data" chunk (and WavPack header should follow). If
-    // the first chunk is not a RIFF, then we assume a "raw" WavPack file and
-    // the WavPack header must be first.
-
-        while (1) {
-
-            if (wpc->reader->read_bytes (wpc->wv_in, &ChunkHeader, sizeof (ChunkHeader)) !=
-                sizeof (ChunkHeader)) {
-                    if (error) strcpy (error, "not a valid WavPack file!");
-                    return WavpackCloseFile (wpc);
-            }
-            else {
-                if (wpc->open_flags & OPEN_WRAPPER) {
-                    wpc->wrapper_data = realloc (wpc->wrapper_data, wpc->wrapper_bytes + sizeof (ChunkHeader));
-                    memcpy (wpc->wrapper_data + wpc->wrapper_bytes, &ChunkHeader, sizeof (ChunkHeader));
-                    wpc->wrapper_bytes += sizeof (ChunkHeader);
-                }
-
-                little_endian_to_native (&ChunkHeader, ChunkHeaderFormat);
-
-                if (!strncmp (ChunkHeader.ckID, "fmt ", 4)) {
-
-                    if (ChunkHeader.ckSize < sizeof (wavhdr) ||
-                        wpc->reader->read_bytes (wpc->wv_in, &wavhdr, sizeof (wavhdr)) != sizeof (wavhdr)) {
-                            if (error) strcpy (error, "not a valid WavPack file!");
-                            return WavpackCloseFile (wpc);
-                    }
-                    else if (wpc->open_flags & OPEN_WRAPPER) {
-                        wpc->wrapper_data = realloc (wpc->wrapper_data, wpc->wrapper_bytes + sizeof (wavhdr));
-                        memcpy (wpc->wrapper_data + wpc->wrapper_bytes, &wavhdr, sizeof (wavhdr));
-                        wpc->wrapper_bytes += sizeof (wavhdr);
-                    }
-
-                    little_endian_to_native (&wavhdr, WaveHeader3Format);
-
-                    if (ChunkHeader.ckSize > sizeof (wavhdr)) {
-                        uint32_t bytes_to_skip = (ChunkHeader.ckSize + 1 - sizeof (wavhdr)) & ~1L;
-
-                        if (bytes_to_skip > 1024 * 1024) {
-                            if (error) strcpy (error, "not a valid WavPack file!");
-                            return WavpackCloseFile (wpc);
-                        }
-
-                        if (wpc->open_flags & OPEN_WRAPPER) {
-                            wpc->wrapper_data = realloc (wpc->wrapper_data, wpc->wrapper_bytes + bytes_to_skip);
-                            wpc->reader->read_bytes (wpc->wv_in, wpc->wrapper_data + wpc->wrapper_bytes, bytes_to_skip);
-                            wpc->wrapper_bytes += bytes_to_skip;
-                        }
-                        else {
-                            unsigned char *temp = malloc (bytes_to_skip);
-                            wpc->reader->read_bytes (wpc->wv_in, temp, bytes_to_skip);
-                            free (temp);
-                        }
-                    }
-                }
-                else if (!strncmp (ChunkHeader.ckID, "data", 4))
-                    break;
-                else if ((ChunkHeader.ckSize + 1) & ~1L) {
-                    uint32_t bytes_to_skip = (ChunkHeader.ckSize + 1) & ~1L;
-
-                    if (bytes_to_skip > 1024 * 1024) {
-                        if (error) strcpy (error, "not a valid WavPack file!");
-                        return WavpackCloseFile (wpc);
-                    }
-
-                    if (wpc->open_flags & OPEN_WRAPPER) {
-                        wpc->wrapper_data = realloc (wpc->wrapper_data, wpc->wrapper_bytes + bytes_to_skip);
-                        wpc->reader->read_bytes (wpc->wv_in, wpc->wrapper_data + wpc->wrapper_bytes, bytes_to_skip);
-                        wpc->wrapper_bytes += bytes_to_skip;
-                    }
-                    else {
-                        unsigned char *temp = malloc (bytes_to_skip);
-                        wpc->reader->read_bytes (wpc->wv_in, temp, bytes_to_skip);
-                        free (temp);
-                    }
-                }
-            }
-        }
-    }
-    else {
-        if (error) strcpy (error, "not a valid WavPack file!");
-        return WavpackCloseFile (wpc);
-    }
-
-    if (wavhdr.FormatTag != 1 || !wavhdr.NumChannels || wavhdr.NumChannels > 2 ||
-        !wavhdr.SampleRate || wavhdr.BitsPerSample < 16 || wavhdr.BitsPerSample > 24 ||
-        wavhdr.BlockAlign / wavhdr.NumChannels > 3 || wavhdr.BlockAlign % wavhdr.NumChannels ||
-        wavhdr.BlockAlign / wavhdr.NumChannels < (wavhdr.BitsPerSample + 7) / 8) {
-            if (error) strcpy (error, "not a valid WavPack file!");
-            return WavpackCloseFile (wpc);
-    }
-
-    wpc->total_samples = ChunkHeader.ckSize / wavhdr.NumChannels /
-        ((wavhdr.BitsPerSample > 16) ? 3 : 2);
-
-    if (wpc->reader->read_bytes (wpc->wv_in, &wphdr, 10) != 10) {
-        if (error) strcpy (error, "not a valid WavPack file!");
-        return WavpackCloseFile (wpc);
-    }
-
-    if (((char *) &wphdr) [8] == 2 && (wpc->reader->read_bytes (wpc->wv_in, ((char *) &wphdr) + 10, 2) != 2)) {
-        if (error) strcpy (error, "not a valid WavPack file!");
-        return WavpackCloseFile (wpc);
-    }
-    else if (((char *) &wphdr) [8] == 3 && (wpc->reader->read_bytes (wpc->wv_in, ((char *) &wphdr) + 10,
-        sizeof (wphdr) - 10) != sizeof (wphdr) - 10)) {
-            if (error) strcpy (error, "not a valid WavPack file!");
-            return WavpackCloseFile (wpc);
-    }
-
-    little_endian_to_native (&wphdr, WavpackHeader3Format);
-
-    // make sure this is a version we know about
-
-    if (strncmp (wphdr.ckID, "wvpk", 4) || wphdr.version < 1 || wphdr.version > 3) {
-        if (error) strcpy (error, "not a valid WavPack file!");
-        return WavpackCloseFile (wpc);
-    }
-
-    // Because I ran out of flag bits in the WavPack header, an amazingly ugly
-    // kludge was forced upon me! This code takes care of preparing the flags
-    // field for internal use and checking for unknown formats we can't decode
-
-    if (wphdr.version == 3) {
-
-        if (wphdr.flags & EXTREME_DECORR) {
-
-            if ((wphdr.flags & NOT_STORED_FLAGS) ||
-                ((wphdr.bits) &&
-                (((wphdr.flags & NEW_HIGH_FLAG) &&
-                (wphdr.flags & (FAST_FLAG | HIGH_FLAG))) ||
-                (wphdr.flags & CROSS_DECORR)))) {
-                    if (error) strcpy (error, "not a valid WavPack file!");
-                    return WavpackCloseFile (wpc);
-            }
-
-            if (wphdr.flags & CANCEL_EXTREME)
-                wphdr.flags &= ~(EXTREME_DECORR | CANCEL_EXTREME);
-        }
-        else
-            wphdr.flags &= ~CROSS_DECORR;
-    }
-
-    // check to see if we should look for a "correction" file, and if so try
-    // to open it for reading, then set WVC_FLAG accordingly
-
-    if (wpc->wvc_in && wphdr.version == 3 && wphdr.bits && (wphdr.flags & NEW_HIGH_FLAG)) {
-        wpc->file2len = wpc->reader->get_length (wpc->wvc_in);
-        wphdr.flags |= WVC_FLAG;
-        wpc->wvc_flag = TRUE;
-    }
-    else
-        wphdr.flags &= ~WVC_FLAG;
-
-    // check WavPack version to handle special requirements of versions
-    // before 3.0 that had smaller headers
-
-    if (wphdr.version < 3) {
-        wphdr.total_samples = wpc->total_samples;
-        wphdr.flags = wavhdr.NumChannels == 1 ? MONO_FLAG : 0;
-        wphdr.shift = 16 - wavhdr.BitsPerSample;
-
-        if (wphdr.version == 1)
-            wphdr.bits = 0;
-    }
-
-    wpc->config.sample_rate = wavhdr.SampleRate;
-    wpc->config.num_channels = wavhdr.NumChannels;
-    wpc->config.channel_mask = 5 - wavhdr.NumChannels;
-
-    if (wphdr.flags & MONO_FLAG)
-        wpc->config.flags |= CONFIG_MONO_FLAG;
-
-    if (wphdr.flags & EXTREME_DECORR)
-        wpc->config.flags |= CONFIG_HIGH_FLAG;
-
-    if (wphdr.bits) {
-        if (wphdr.flags & NEW_HIGH_FLAG)
-            wpc->config.flags |= CONFIG_HYBRID_FLAG;
-        else
-            wpc->config.flags |= CONFIG_LOSSY_MODE;
-    }
-    else if (!(wphdr.flags & HIGH_FLAG))
-        wpc->config.flags |= CONFIG_FAST_FLAG;
-
-    wpc->config.bytes_per_sample = (wphdr.flags & BYTES_3) ? 3 : 2;
-    wpc->config.bits_per_sample = wavhdr.BitsPerSample;
-
-    memcpy (&wps->wphdr, &wphdr, sizeof (wphdr));
-    wps->wvbits.bufsiz = wps->wvcbits.bufsiz = 1024 * 1024;
-    return wpc;
-}
-
-// return currently decoded sample index
-
-uint32_t get_sample_index3 (WavpackContext *wpc)
-{
-    WavpackStream3 *wps = (WavpackStream3 *) wpc->stream3;
-
-    return (wps) ? wps->sample_index : (uint32_t) -1;
-}
-
-int get_version3 (WavpackContext *wpc)
-{
-    WavpackStream3 *wps = (WavpackStream3 *) wpc->stream3;
-
-    return (wps) ? wps->wphdr.version : 0;
-}
-
-void free_stream3 (WavpackContext *wpc)
-{
-    WavpackStream3 *wps = (WavpackStream3 *) wpc->stream3;
-
-    if (wps) {
-#ifndef NO_SEEKING
-        if (wps->unpack_data)
-            free (wps->unpack_data);
-#endif
-        if (wps->wphdr.flags & WVC_FLAG)
-            bs_close_read3 (&wps->wvcbits);
-
-        bs_close_read3 (&wps->wvbits);
-
-        free (wps);
-    }
-}
-
-static void bs_read3 (Bitstream3 *bs)
-{
-    uint32_t bytes_read;
-
-    bytes_read = bs->reader->read_bytes (bs->id, bs->buf, bs->bufsiz);
-    bs->end = bs->buf + bytes_read;
-    bs->fpos += bytes_read;
-
-    if (bs->end == bs->buf) {
-        memset (bs->buf, -1, bs->bufsiz);
-        bs->end += bs->bufsiz;
-    }
-
-    bs->ptr = bs->buf;
-}
-
-// Open the specified BitStream and associate with the specified file. The
-// "bufsiz" field of the structure must be preset with the desired buffer
-// size and the file's read pointer must be set to where the desired bit
-// data is located.  A return value of TRUE indicates an error in
-// allocating buffer space.
-
-static int bs_open_read3 (Bitstream3 *bs, WavpackStreamReader *reader, void *id)
-{
-    bs->fpos = (bs->reader = reader)->get_pos (bs->id = id);
-
-    if (!bs->buf)
-        bs->buf = (unsigned char *) malloc (bs->bufsiz);
-
-    bs->end = bs->buf + bs->bufsiz;
-    bs->ptr = bs->end - 1;
-    bs->sr = bs->bc = 0;
-    bs->error = bs->buf ? 0 : 1;
-    bs->wrap = bs_read3;
-    return bs->error;
-}
-
-#ifndef NO_SEEKING
-
-// This function is called after a call to unpack_restore() has restored
-// the BitStream structure to a previous state and causes any required data
-// to be read from the file. This function is NOT supported for overlapped
-// operation.
-
-static void bs_restore3 (Bitstream3 *bs)
-{
-    uint32_t bytes_to_read = (uint32_t)(bs->end - bs->ptr - 1), bytes_read;
-
-    bs->reader->set_pos_abs (bs->id, bs->fpos - bytes_to_read);
-
-    if (bytes_to_read > 0) {
-
-        bytes_read = bs->reader->read_bytes (bs->id, bs->ptr + 1, bytes_to_read);
-
-        if (bytes_to_read != bytes_read)
-            bs->end = bs->ptr + 1 + bytes_read;
-    }
-}
-
-#endif
-
-// This function is called to release any resources used by the BitStream
-// and position the file pointer to the first byte past the read bits.
-
-static void bs_close_read3 (Bitstream3 *bs)
-{
-    if (bs->buf) {
-        free (bs->buf);
-        CLEAR (*bs);
-    }
-}
-
-static uint32_t bs_unused_bytes (Bitstream3 *bs)
-{
-    if (bs->bc < 8) {
-        bs->bc += 8;
-        bs->ptr++;
-    }
-
-    return (uint32_t)(bs->end - bs->ptr);
-}
-
-static unsigned char *bs_unused_data (Bitstream3 *bs)
-{
-    if (bs->bc < 8) {
-        bs->bc += 8;
-        bs->ptr++;
-    }
-
-    return bs->ptr;
-}
-
-#ifndef NO_UNPACK
+static uint32_t bs_unused_bytes (Bitstream3 *bs);
+static unsigned char *bs_unused_data (Bitstream3 *bs);
+static void init_words3 (WavpackStream3 *wps);
 
 //////////////////////////////// local macros /////////////////////////////////
 
@@ -426,13 +61,13 @@ static const signed char extreme_terms [] = { 1,1,1,2,4,-1,1,2,3,6,-2,8,5,7,4,1,
 static const signed char default_terms [] = { 1,1,1,-1,2,1,-2 };
 static const signed char simple_terms []  = { 1,1,1,1 };
 
+///////////////////////////// executable code ////////////////////////////////
+
 // This function initializes everything required to unpack WavPack
 // bitstreams and must be called before any unpacking is performed. Note
 // that the (WavpackHeader3 *) in the WavpackStream3 struct must be valid.
 
-static void init_words3 (WavpackStream3 *wps);
-
-static void unpack_init3 (WavpackStream3 *wps)
+void unpack_init3 (WavpackStream3 *wps)
 {
     int flags = wps->wphdr.flags;
     struct decorr_pass *dpp;
@@ -461,9 +96,6 @@ static void unpack_init3 (WavpackStream3 *wps)
 
 #ifndef NO_SEEKING
 
-#define SAVE(destin, item) { memcpy (destin, &item, sizeof (item)); destin = (char *) destin + sizeof (item); }
-#define RESTORE(item, source) { memcpy (&item, source, sizeof (item)); source = (char *) source + sizeof (item); }
-
 // This function returns the size (in bytes) required to save the unpacking
 // context. Note that the (WavpackHeader3 *) in the WavpackStream3 struct
 // must be valid.
@@ -606,173 +238,6 @@ static void *unpack_save (WavpackStream3 *wps, void *destin)
     return destin;
 }
 
-// This function restores the unpacking context from the specified pointer
-// and returns the updated pointer. After this call, unpack_samples() will
-// continue where it left off immediately before unpack_save() was called.
-// If the WavPack files and bitstreams might have been closed and reopened,
-// then the "keep_resources" flag should be set to avoid using the "old"
-// resources that were originally saved (and are probably now invalid).
-
-static void *unpack_restore (WavpackStream3 *wps, void *source, int keep_resources)
-{
-    int flags = wps->wphdr.flags, tcount;
-    struct decorr_pass *dpp;
-    FILE *temp_file;
-    unsigned char *temp_buf;
-
-    unpack_init3 (wps);
-    temp_file = wps->wvbits.id;
-    temp_buf = wps->wvbits.buf;
-    RESTORE (wps->wvbits, source);
-
-    if (keep_resources) {
-        wps->wvbits.id = temp_file;
-        wps->wvbits.ptr += temp_buf - wps->wvbits.buf;
-        wps->wvbits.end += temp_buf - wps->wvbits.buf;
-        wps->wvbits.buf = temp_buf;
-    }
-
-    bs_restore3 (&wps->wvbits);
-
-    if (flags & WVC_FLAG) {
-        temp_file = wps->wvcbits.id;
-        temp_buf = wps->wvcbits.buf;
-        RESTORE (wps->wvcbits, source);
-
-        if (keep_resources) {
-            wps->wvcbits.id = temp_file;
-            wps->wvcbits.ptr += temp_buf - wps->wvcbits.buf;
-            wps->wvcbits.end += temp_buf - wps->wvcbits.buf;
-            wps->wvcbits.buf = temp_buf;
-        }
-
-        bs_restore3 (&wps->wvcbits);
-    }
-
-    if (wps->wphdr.version == 3) {
-        if (wps->wphdr.bits) {
-            RESTORE (wps->w4, source);
-        }
-        else {
-            RESTORE (wps->w1, source);
-        }
-
-        RESTORE (wps->w3, source);
-        RESTORE (wps->dc.crc, source);
-    }
-    else
-        RESTORE (wps->w2, source);
-
-    if (wps->wphdr.bits) {
-        RESTORE (wps->dc.error, source);
-    }
-    else {
-        RESTORE (wps->dc.sum_level, source);
-        RESTORE (wps->dc.left_level, source);
-        RESTORE (wps->dc.right_level, source);
-        RESTORE (wps->dc.diff_level, source);
-    }
-
-    if (flags & OVER_20) {
-        RESTORE (wps->dc.last_extra_bits, source);
-        RESTORE (wps->dc.extra_bits_count, source);
-    }
-
-    if (!(flags & EXTREME_DECORR)) {
-        RESTORE (wps->dc.sample, source);
-        RESTORE (wps->dc.weight, source);
-    }
-
-    if (flags & (HIGH_FLAG | NEW_HIGH_FLAG))
-        for (tcount = wps->num_terms, dpp = wps->decorr_passes; tcount--; dpp++)
-            if (dpp->term > 0) {
-                int count = dpp->term;
-                int index = wps->dc.m;
-
-                RESTORE (dpp->weight_A, source);
-
-                while (count--) {
-                    RESTORE (dpp->samples_A [index], source);
-                    index = (index + 1) & (MAX_TERM - 1);
-                }
-
-                if (!(flags & MONO_FLAG)) {
-                    count = dpp->term;
-                    index = wps->dc.m;
-
-                    RESTORE (dpp->weight_B, source);
-
-                    while (count--) {
-                        RESTORE (dpp->samples_B [index], source);
-                        index = (index + 1) & (MAX_TERM - 1);
-                    }
-                }
-            }
-            else {
-                RESTORE (dpp->weight_A, source);
-                RESTORE (dpp->weight_B, source);
-                RESTORE (dpp->samples_A [0], source);
-                RESTORE (dpp->samples_B [0], source);
-            }
-
-    return source;
-}
-
-// This is an extension for WavpackSeekSample (). Note that because WavPack
-// files created prior to version 4.0 are not inherently seekable, this
-// function could take a long time if a forward seek is requested to an
-// area that has not been played (or seeked through) yet.
-
-
-int seek_sample3 (WavpackContext *wpc, uint32_t desired_index)
-{
-    int points_index = desired_index / ((wpc->total_samples >> 8) + 1);
-    WavpackStream3 *wps = (WavpackStream3 *) wpc->stream3;
-
-    if (desired_index >= wpc->total_samples)
-        return FALSE;
-
-    while (points_index)
-        if (wps->index_points [points_index].saved &&
-            wps->index_points [points_index].sample_index <= desired_index)
-                break;
-        else
-            points_index--;
-
-    if (wps->index_points [points_index].saved)
-        if (wps->index_points [points_index].sample_index > wps->sample_index ||
-            wps->sample_index > desired_index) {
-                wps->sample_index = wps->index_points [points_index].sample_index;
-                unpack_restore (wps, wps->unpack_data + points_index * wps->unpack_size, TRUE);
-        }
-
-    if (desired_index > wps->sample_index) {
-        int32_t *buffer = (int32_t *) malloc (1024 * (wps->wphdr.flags & MONO_FLAG ? 4 : 8));
-        uint32_t samples_to_skip = desired_index - wps->sample_index;
-
-        while (1) {
-            if (samples_to_skip > 1024) {
-                if (unpack_samples3 (wpc, buffer, 1024) == 1024)
-                    samples_to_skip -= 1024;
-                else
-                    break;
-            }
-            else {
-                samples_to_skip -= unpack_samples3 (wpc, buffer, samples_to_skip);
-                break;
-            }
-        }
-
-        free (buffer);
-
-        if (samples_to_skip)
-            return FALSE;
-    }
-
-    return TRUE;
-}
-
-
 #endif
 
 // This monster actually unpacks the WavPack bitstream(s) into the specified
@@ -1753,12 +1218,6 @@ int32_t unpack_samples3 (WavpackContext *wpc, int32_t *buffer, uint32_t sample_c
     return i;
 }
 
-///////////////////////////// local table storage ////////////////////////////
-
-extern const uint32_t bitset [];
-extern const uint32_t bitmask [];
-extern const char nbits_table [];
-
 // This function initializes everything required to receive words with this
 // module and must be called BEFORE any other function in this module.
 
@@ -1775,18 +1234,6 @@ static void init_words3 (WavpackStream3 *wps)
         wps->w4.bitrate = (wps->wphdr.bits / 2) - 768;
 }
 
-// This macro counts the number of bits that are required to specify the
-// unsigned 32-bit value, counting from the LSB to the most significant bit
-// that is set. Return range is 0 - 32.
-
-#define count_bits(av) ( \
- (av) < (1 << 8) ? nbits_table [av] : \
-  ( \
-   (av) < (1L << 16) ? nbits_table [(av) >> 8] + 8 : \
-   ((av) < (1L << 24) ? nbits_table [(av) >> 16] + 16 : nbits_table [(av) >> 24] + 24) \
-  ) \
-)
-
 static int32_t FASTCALL get_word1 (WavpackStream3 *wps, int chan)
 {
     uint32_t tmp1, tmp2, avalue;
@@ -2191,5 +1638,60 @@ static int FASTCALL _log2 (uint32_t avalue)
     }
 }
 
-#endif
+static void bs_read3 (Bitstream3 *bs)
+{
+    uint32_t bytes_read;
+
+    bytes_read = bs->reader->read_bytes (bs->id, bs->buf, bs->bufsiz);
+    bs->end = bs->buf + bytes_read;
+    bs->fpos += bytes_read;
+
+    if (bs->end == bs->buf) {
+        memset (bs->buf, -1, bs->bufsiz);
+        bs->end += bs->bufsiz;
+    }
+
+    bs->ptr = bs->buf;
+}
+
+// Open the specified BitStream and associate with the specified file. The
+// "bufsiz" field of the structure must be preset with the desired buffer
+// size and the file's read pointer must be set to where the desired bit
+// data is located.  A return value of TRUE indicates an error in
+// allocating buffer space.
+
+static int bs_open_read3 (Bitstream3 *bs, WavpackStreamReader *reader, void *id)
+{
+    bs->fpos = (bs->reader = reader)->get_pos (bs->id = id);
+
+    if (!bs->buf)
+        bs->buf = (unsigned char *) malloc (bs->bufsiz);
+
+    bs->end = bs->buf + bs->bufsiz;
+    bs->ptr = bs->end - 1;
+    bs->sr = bs->bc = 0;
+    bs->error = bs->buf ? 0 : 1;
+    bs->wrap = bs_read3;
+    return bs->error;
+}
+
+static uint32_t bs_unused_bytes (Bitstream3 *bs)
+{
+    if (bs->bc < 8) {
+        bs->bc += 8;
+        bs->ptr++;
+    }
+
+    return (uint32_t)(bs->end - bs->ptr);
+}
+
+static unsigned char *bs_unused_data (Bitstream3 *bs)
+{
+    if (bs->bc < 8) {
+        bs->bc += 8;
+        bs->ptr++;
+    }
+
+    return bs->ptr;
+}
 
